package org.illeva.remcom;

import java.util.*;
import java.io.*;
/**
  * E' la classe che esegue i processi batch.
  */
public class OsRunner
{
	// Data members
	/** Nome logico del processo da eseguire */
	private String dName;
	/**
	  * Nome fisico del processo da eseguire. Si tratta di un vettore di stringhe perch il nome fisico del processo
	  * pu essere la composizione di pi stringhe
	  */
	private Vector dCmd=new Vector();
	/**
	  * E' la collezione di coppie chiave/valore che costituiscono
	  * le variabili d'ambiente con le quali
	  * il processo in eseguzione pu interagire.
	  */
	private HashMap dEnv=new HashMap();
	/** E' la directory di lavoro del processo. */
	private File dWorkDir;
	// Function
	/**
	  * Costruisce un'istanza della classe attribuendogli un nome logico.
	  */
	public OsRunner(String aName)
	{
		dName=aName;
	} // end constructor
	/** Restituisce il nome logico del processo. */
	public String getName() {return dName;}
	/** Imposta la directory di lavoro. */
	public void setWorkDir(String aWorkDir)
	{
		dWorkDir=new File(aWorkDir);
	} // end method
	// public void addCmd(String aCmd) {;}
	/** Imposta un comando fisico da eseguire */
	public void addCmdList(String aList)
	{
		StringTokenizer vToken=new StringTokenizer(aList,",");
		while(vToken.hasMoreElements())
			dCmd.add(vToken.nextToken());
	} // end method
	public void addEnv(String aKey,String aValue)
	{
		dEnv.put(aKey,aValue);
	} // end method
	/** Aggiunge le variabili d'ambiente del processo. */
	public void addEnvList(String aList)
	{
		StringTokenizer vToken=new StringTokenizer(aList,",");
		while(vToken.hasMoreElements())
		{
			String vTemp=vToken.nextToken();
			int vPos=vTemp.indexOf("=");
			addEnv(vTemp.substring(0,vPos),vTemp.substring(vPos+1));
		} // end while
	} // end method
	/** Esegue il processo batch. */
	public void run()
	{
		String[] vCmd=null;
		String[] vEnv=null;
		/* Trasforma il vettore di stringhe che rappresenta il comando da lanciare
		 in un array di stringhe. */
		if(dCmd.size()>0)
		{
			vCmd=new String[dCmd.size()];
			for(int i=0; i<dCmd.size(); i++)
				vCmd[i]=(String)(dCmd.elementAt(i));
		} // end if
		/* Trasforma la collection di coppie-valore in un array di stringhe. */
		if(dEnv.size()>0)
		{
			vEnv=new String[dEnv.size()];
			Iterator i=dEnv.keySet().iterator();
			int j=0;
			while(i.hasNext())
			{
				String vTemp=(String)(i.next());
				vEnv[j++]=vTemp+"="+((String)(dEnv.get(vTemp)));
			} // end while
		} // end if
		try
		{
		Process vChild=Runtime.getRuntime().exec(vCmd,vEnv,dWorkDir);
		}
		catch(IOException anEx)
		{
			anEx.printStackTrace(System.err);
		} // end catch
	} // end method
} // end class