package org.illeva.remcom.cmdsock;

import java.io.*;
import java.util.*;
import java.net.*;

/**
  * E' il thread che rimane in ascolto su di una porta per ricevere i comandi che verranno trasmessi
  * all'applicazione target.
  */
public class MainSocket extends Thread
{
	// Data members
	/** E' il flag per gestire l'esistenza del thread. */
	private boolean dExists=true;
	/** E' il flag che attiva il thread oppure ne sospende le attivit. */
	private boolean dActive=true;
	/** E' il socket che ascolta una porta in attesa di comandi. */
	private ServerSocket dSock=null;
	/** File di configurazione per la configurazione dell'applicazione. */
	private Properties dProp=new Properties();
	/** E' il tempo di sleep tra l'attesa di un comando ed il successivo. */
	long dSleep=3000;
	/** E' l'applicazione target a cui i comandi andranno inviati. */
	private SocketListener dApp=null;
	// Funtction members
	/**
	  * Imposta l'omonimo flag. Se attivo l'applicazione  in grado di ricevere comandi remoti.
	  */
	public synchronized void setActive(boolean aFlag)
	{
		dActive=aFlag;
	} // end method
	/**
	  * Imposta l'monimo flag. Se attivo il thread esiste.

	  */
	public synchronized void setExists(boolean aFlag)
	{
		dExists=aFlag;
	} // end method
	/**
	  * Costruisce un thread d'ascolto che ha come parametro un'applicazione target.
	  */
	public MainSocket(SocketListener anApp)
	{
		dApp=anApp;
		setPriority(MIN_PRIORITY);
		try
		{
			dProp.load(new FileInputStream("../config/MainSocket.properties"));
			dSock=new ServerSocket(
				new Integer(dProp.getProperty("port","1304")).intValue()
				,new Integer(dProp.getProperty("backlog","3")).intValue()
			);
			dSleep=new Long(dProp.getProperty("sleep","5000")).longValue();
			dSock.setSoTimeout((int)dSleep);
		} // end try
		catch(IOException anEx)
		{
			System.err.println(anEx);
		} // end catch
	} // end method
	/**
	  * E' il metodo vero e proprio del thread. Non appena viene ricevuto un comando, un socket viene associato
	  * alla richiesta ed inizia il colloquio vero e proprio fra Client e Server.
	  */
	public void run()
	{
	  /* Ciclo che regola l'sistenza del thread */
		while(dExists)
		{
		  /* Ciclo che regola l'ascolto o meno sul canale del thread stesso */
			while(dActive)
			{
				Socket vSock=null;
				try
				{
					// System.out.println("Prima della accept...");
					vSock=dSock.accept();
					new ChildSocket(vSock,dApp).start();
					System.out.println("Ho creato un Child...");
				} // end try
				catch(SocketTimeoutException anEx)
				{
					// System.out.print(".");
				} // end catch
				catch(IOException anEx)
				{
					System.err.println(anEx);
				} // end catch
			} // end while
			/* Se l'ascolto sul canale  disattivato il thread dorme per un po' di tempo */
			try
			{
				// System.out.println("Sto dormendo...");
				sleep(dSleep);
			} // end try
			catch(InterruptedException anEx)
			{
				System.out.println("MainSocket wake up...");
			} // end catch
		} // end while
	} // end method
} // end class