package org.illeva.remcom.gui;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import org.illeva.remcom.cmdsock.*;
import org.illeva.remcom.OsRunner;

/**
  * E' l'interfaccia del programma server.
  */
public class MainWin extends Frame implements ActionListener,WindowListener,SocketListener
{
	// GUI elements
	/**
	  * E' la lista dei comandi che possono essere lanciati direttamente sul server utilizzando l'interfaccia.
	  */
	java.awt.List dCmdList=new java.awt.List();
	/** Esegue il comando scelto. */
	Button dCmdButton=new Button("Execute");
	/** sospende ed attiva la possibilit di eseguire i comandi da remoto */
	Button dSocketButton=new Button("Stop socket");
	// ServetSocket elements
	MainSocket dDaemon=null;
	HashMap dProcMap=new HashMap();
	public MainWin()
	{
		super("Remote Commander V1.0");
		add(dSocketButton,BorderLayout.WEST);
		add(dCmdList,BorderLayout.CENTER);
		add(dCmdButton,BorderLayout.EAST);
		dCmdButton.addActionListener(this);
		dSocketButton.addActionListener(this);
		addWindowListener(this);
		loadProcess();
		refreshList();
		dDaemon=new MainSocket(this);
		dDaemon.start();
	} // end constructor
	/** Processa i comandi remoti della GUI */
	public void processWinCommand(SocketEvent anEvent)
	{
		switch(anEvent.getCommand())
		{
			case SocketEvent.WIN_CLOSE:
				closeApp();
			break;
			case SocketEvent.WIN_HIDE:
				hide();
			break;
			case SocketEvent.WIN_SHOW:
				show();
			break;
		} // end switch
	} // end method
	/** Processa i comandi remoti di carattere applicativo. */
	public String processApplicationCommand(SocketEvent anEvent)
	{
		String vRc="Executed command";
		switch(anEvent.getCommand())
		{
			case SocketEvent.APP_CMDLIST:
				vRc="CmdList(Win(Close),Win(Hide),Win(Show)";
				for(int i=0; i<dCmdList.getItemCount(); i++)
					vRc+=",App("+dCmdList.getItem(i).trim()+")";
				vRc+=")";
			break;
			case SocketEvent.APP_PARAM:
				execute(anEvent.getParam());
			break;
		} // end switch
		return vRc;
	} // end method
	public void actionPerformed(ActionEvent anEvent)
	{
		// System.out.println(anEvent);
		if(anEvent.getActionCommand().equals("Stop socket"))
		{
			dDaemon.setActive(false);
			dSocketButton.setLabel("Start socket");
		} // end if
		else if(anEvent.getActionCommand().equals("Start socket"))
		{
			dDaemon.setActive(true);
			dSocketButton.setLabel("Stop socket");
		} // end else if
		else if(anEvent.getActionCommand().equals("Execute"))
		{
			int vPos=dCmdList.getSelectedIndex();
			if(vPos!=-1)
				execute(dCmdList.getItem(vPos).trim());
		} // end else if
			else
				System.out.println("Unknow WinCmd: "+anEvent.getActionCommand());
	} // end method
	public void windowActivated(WindowEvent anEvent)
	{
	} // end method
	public void windowClosed(WindowEvent anEvent)
	{
	} // end method
	public void windowClosing(WindowEvent anEvent)
	{
		closeApp();
	} // end method
	public void windowDeactivated(WindowEvent anEvent)
	{
	} // end method
	public void windowDeiconified(WindowEvent anEvent)
	{
	} // end method
	public void windowIconified(WindowEvent anEvent)
	{
	} // end method
	public void windowOpened(WindowEvent anEvent)
	{
	} // end method
	private void closeApp()
	{
		dDaemon.setActive(false);
		dDaemon.setExists(false);
		try
		{
		dDaemon.join();
		} // end try
		catch(InterruptedException anEx)
		{
			System.err.println(anEx);
		} // end catch
		System.exit(0);
	} // end method
	/**
	  * Carica i processi che possono essere eseguiti.
	  */
	private void loadProcess()
	{
		OsRunner vObj=null;
		try
		{
				BufferedReader vIs=new BufferedReader(new FileReader("../Config/process.txt"));
				String vLine=vIs.readLine();
				while(vLine!=null)
				{
					if(!vLine.startsWith("#"))
					{
						if(vLine.startsWith("ProcName="))
						{
							if(vObj!=null)
								dProcMap.put(vObj.getName(),vObj);
							vObj=new OsRunner(vLine.substring(9));
						} // end if
						else if(vLine.startsWith("CmdList="))
						{
							vObj.addCmdList(vLine.substring(8));
						} // end else if
						else if(vLine.startsWith("EnvList="))
						{
							vObj.addEnvList(vLine.substring(8));
						} // end else if
						else if(vLine.startsWith("WorkDir="))
						{
							vObj.setWorkDir(vLine.substring(8));
						} // end else if
					} // end if
					vLine=vIs.readLine();
				} // end while
				if(vObj!=null)
					dProcMap.put(vObj.getName(),vObj);
		} // end try
		catch(FileNotFoundException anEx)
		{
			System.err.println(anEx);
		} // end catch
		catch(IOException anEx)
		{
			System.err.println(anEx);
		} // end catch
	} // end method
	/** Visualizza i comandi nella lista. */
	private void refreshList()
	{
		Iterator i=dProcMap.keySet().iterator();
		while(i.hasNext())
			dCmdList.add((String)(i.next()));
	} // end method
	/** Esegue un comando scelto dalla lista oppure richiesto via remoto. */
	private void execute(String aCmd)
	{
		OsRunner vTemp=(OsRunner)(dProcMap.get(aCmd));
		if(vTemp!=null)
					vTemp.run();
	} // end method
} // end class